package model;

import java.io.File;

import model.ontology.Node;
import model.ontology.Ontology;
import view.components.SynsetCellRender;
import edu.smu.tspell.wordnet.NounSynset;
import edu.smu.tspell.wordnet.Synset;
import edu.smu.tspell.wordnet.SynsetType;
import edu.smu.tspell.wordnet.WordNetDatabase;
import edu.smu.tspell.wordnet.impl.file.SynsetFactory;
import edu.smu.tspell.wordnet.impl.file.SynsetPointer;
import edu.smu.tspell.wordnet.impl.file.synset.VerbReferenceSynset;

/**
 * Class that controls the WordNet database
 * @author gonta
 *
 */
public class WordNet {

	/**
	 * This method obtains all of synonymous of type NOUN of a word
	 * @param word word requested
	 * @return
	 */
	public static Synset[] getSynonyms( String word ){
		
		word = word.replace(" ", "_");
		/** Directory of WordNet */
		System.setProperty("wordnet.database.dir", "lib"+File.separator+"WordNet-3.0"+File.separator+"dict"+File.separator);
		
		WordNetDatabase database = WordNetDatabase.getFileInstance();
		
		Synset[] synsets = database.getSynsets( word, SynsetType.NOUN );
		
		return synsets;
	}
	
	/**
	 * This method obtains all of synonymous of a specified type for a word
	 * @param word word requested
	 * @param morphoTag POS-Tag of synonymous to find
	 * @return
	 */
	public static Synset[] getSynonymsBySynsetType( String word , String morphoTag){
		
		SynsetType st  = SynsetType.NOUN;
		if(morphoTag.equals("VERB"))
			st = SynsetType.VERB;
		
		
		word = word.replace(" ", "_");
		
		System.setProperty("wordnet.database.dir", "lib"+File.separator+"WordNet-3.0"+File.separator+"dict"+File.separator);
		
		WordNetDatabase database = WordNetDatabase.getFileInstance();
		
		Synset[] synsets = database.getSynsets( word, st);
			
		return synsets;
	}
	
	/**
	 * This method obtains a synset corresponding to a identifier and pos-tag
	 * @param id synset id
	 * @param morphoTag POS-Tag
	 * @return
	 */
	public static Synset getSynset(int id, String morphoTag) {		
		System.setProperty("wordnet.database.dir", "lib"+File.separator+"WordNet-3.0"+File.separator+"dict"+File.separator);
		
		SynsetType st  = SynsetType.NOUN;
		if(morphoTag.equals("VERB"))
			st = SynsetType.VERB;
		
		SynsetFactory factory = SynsetFactory.getInstance();
		return factory.getSynset( new SynsetPointer( st, id ) );
	}
	
	/**
	 * This method add a sub-hierarchy for a synset to a ontology
	 * @param synset  synset
	 * @param chSynset synset child
	 */
	public static void addTreeForSynset( Synset synset, Synset chSynset) {
		Ontology g = Ontology.getInstance();
		Synset [] hypers = null;
		Node node = null;
		Node chNode = null;
		Synset brotherSynsets [] = null;
		
		if( chSynset == null ) {
			node = g.addNode(SynsetCellRender.formatSynonyms(synset), synset.hashCode(), true);
			Synset tropos[];				
			if( synset.getType().equals(SynsetType.VERB ))
				tropos = ((VerbReferenceSynset) synset).getTroponyms();
			else
				tropos = ((NounSynset) synset).getHyponyms();
			for (Synset troponimo : tropos) {					
				Node nodeTropo = g.addNode(SynsetCellRender.formatSynonyms(troponimo), troponimo.hashCode(),false);
				g.addRelation(node, nodeTropo, false);
			}
		} else {
			node = g.get(new Node(SynsetCellRender.formatSynonyms(synset), synset.hashCode()));
			chNode = g.get(new Node(SynsetCellRender.formatSynonyms(chSynset), chSynset.hashCode()));
			g.addRelation(node, chNode, true);	
			
		}
			
		if( synset.getType().equals( SynsetType.VERB ))
			hypers = ((VerbReferenceSynset) synset).getHypernyms();
		else
			hypers = ((NounSynset) synset).getHypernyms();
		//if(chSynset == null)
		//	System.out.println("numero de hyperonimous de: " + SynsetCellRender.formatSynonyms((VerbReferenceSynset) synset) +" -> " + hypers.length );
		if ( hypers.length == 0)
			if( synset.getType().equals( SynsetType.VERB ))
				g.addRelationRoot(node,1);
			else
				g.addRelationRoot(node,2);
		else {
			for (Synset hyper : hypers) {
				//System.out.println("Hyponymous : " + SynsetCellRender.formatSynonyms((VerbReferenceSynset) hyper));
				Node hyperNode = g.addNode(SynsetCellRender.formatSynonyms(hyper), hyper.hashCode(),false);
				if( chNode == null ) {
					if( synset.getType().equals(SynsetType.VERB ) )
						brotherSynsets = ((VerbReferenceSynset) hyper).getTroponyms();
					else
						brotherSynsets = ((NounSynset) hyper).getHyponyms();
					for (Synset synsetBrother : brotherSynsets) {						
						if(synsetBrother.hashCode() != synset.hashCode()){ //garantindo que eh irmao							
							Node brotherNode = g.addNode(SynsetCellRender.formatSynonyms(synsetBrother), synsetBrother.hashCode(), false);
							g.addRelation(hyperNode, brotherNode, false);
						}							
					}
				}
				addTreeForSynset(hyper, synset);			
			}	
		}	
	}
	
	
	public static void main( String [] args ) {
		
		System.setProperty("wordnet.database.dir", "lib"+File.separator+"WordNet-3.0"+File.separator+"dict"+File.separator);		
		WordNetDatabase database = WordNetDatabase.getFileInstance();
		Synset[] synsets = database.getSynsets( "cancel", SynsetType.VERB );
		for (Synset synset : synsets) {
			System.out.println(SynsetCellRender.formatSynonyms(synset) + " : " + synset.getDefinition());
		}	
//		System.out.println("cuac");
//		for (Synset synset : WordNet.getSynonymsBySynsetType("investigate", "VERB")) {
//			System.out.println(SynsetCellRender.formatSynonyms(synset) + " : " + synset.getDefinition());
//		}
//		
		System.out.println("lol");
		System.out.println(SynsetCellRender.formatSynonyms(WordNet.getSynset(831651, "VERB")) + " : " + WordNet.getSynset(831651, "VERB").getDefinition());
		System.out.println(SynsetCellRender.formatSynonyms(WordNet.getSynset(1009240, "VERB")) + " : " + WordNet.getSynset(1009240, "VERB").getDefinition());
		System.out.println(SynsetCellRender.formatSynonyms(WordNet.getSynset(2118476, "VERB")) + " : " + WordNet.getSynset(2118476, "VERB").getDefinition());
		System.out.println(SynsetCellRender.formatSynonyms(WordNet.getSynset(965871, "VERB")) + " : " + WordNet.getSynset(965871, "VERB").getDefinition());
		
		
	}
}
